@@ -1,9 +1,9 @@
-#!/usr/local/bin/perl
+#!/usr/bin/perl
 #
-#	gentest_ellipsoid.pl
+#       gentest_ellipsoid.pl
 #
-#	Test Ellipsoid.pm module coordinate transformations and
-#	generate test programs.
+#       Test Ellipsoid.pm module coordinate transformations and
+#       generate test programs.
 
 use strict;
 use warnings;
@@ -34,19 +34,19 @@ my $nm    = 1852.0;
 my $kilo  = 1000.0;
 
 # global variables
-my $e = Geo::Ellipsoid->new( units => 'degrees' );
-my $e_pos = Geo::Ellipsoid->new( units => 'degrees' );
-my $e_sym = Geo::Ellipsoid->new( 
-  units => 'degrees', 
-  bearing => 1, 
-  longitude => 1 
+my $e = Geo::Ellipsoid->new( angle_unit => 'degrees' );
+my $e_pos = Geo::Ellipsoid->new( angle_unit => 'degrees' );
+my $e_sym = Geo::Ellipsoid->new(
+  angle_unit => 'degrees',
+  bearing_symmetric => 1,
+  longitude_symmetric => 1
 );
 
 my $outdir = '.';
 #my $outdir = 'Ellipsoid';
 
 # make copy of original set of pre-defined ellipsoids
-my @ellipsoids = keys %Geo::Ellipsoid::ellipsoids;
+my @ellipsoids = sort keys %Geo::Ellipsoid::ellipsoids;
 
 print "Enter test_ellipsoid\n\n";
 
@@ -59,14 +59,14 @@ die("Invalid options: @ARGV") unless GetOptions(
   'o' => \$write,
 );
 
-srand(0);	# set random seed for repeatability
+srand(0);       # set random seed for repeatability
 
 # create directory to hold generated files
 unless( -d $outdir ) {
   mkdir $outdir or die("Can't create directory $outdir: $!");
 }
 
-my @files = qw( 
+my @files = qw(
   load create defaults set scale to at location range bearing displacement
 );
 my %tests;
@@ -93,53 +93,59 @@ sub test_loading_module
   print "Generate loading module tests\n" if $debug;
   my $code = <<EOS;
 BEGIN { use_ok( 'Geo::Ellipsoid' ); }
+
 my \$e = Geo::Ellipsoid->new();
 isa_ok( \$e, 'Geo::Ellipsoid');
-my \$e1 = Geo::Ellipsoid->new( units => 'degrees' );
+
+my \$e1 = Geo::Ellipsoid->new( angle_unit => 'degrees' );
 isa_ok( \$e1, 'Geo::Ellipsoid');
-my \$e2 = Geo::Ellipsoid->new( distance_units => 'foot' );
+
+my \$e2 = Geo::Ellipsoid->new( distance_unit => 'foot' );
 isa_ok( \$e2, 'Geo::Ellipsoid');
-my \$e3 = Geo::Ellipsoid->new( bearing => 1 );
+
+my \$e3 = Geo::Ellipsoid->new( bearing_symmetric => 1 );
 isa_ok( \$e3, 'Geo::Ellipsoid');
-my \$e4 = Geo::Ellipsoid->new( longitude => 1 );
+
+my \$e4 = Geo::Ellipsoid->new( longitude_symmetric => 1 );
 isa_ok( \$e4, 'Geo::Ellipsoid');
 EOS
   push(@{$tests{load}{code}},$code);
   $tests{load}{count} = 6;
-  
-  for my $s ( 
-    qw{ 
-      new 
-      set_units 
-      set_distance_unit 
-      set_ellipsoid 
-      set_custom_ellipsoid 
-      set_longitude_symmetric 
-      set_bearing_symmetric 
+
+  my $t = '';
+  for my $s (
+    qw{
+      new
+      set_angle_unit
+      set_distance_unit
+      set_ellipsoid
+      set_custom_ellipsoid
+      set_longitude_symmetric
+      set_bearing_symmetric
       set_defaults
-      scales 
-      range 
-      bearing 
-      at 
-      to 
-      displacement 
+      scales
+      range
+      bearing
+      at
+      to
+      displacement
       location
-    } 
-  
+    }
+
 ) {
-    my $t = "can_ok( 'Geo::Ellipsoid', '$s' );";
-    push(@{$tests{load}{code}},$t);
+    $t .= "can_ok( 'Geo::Ellipsoid', '$s' );\n";
     $tests{load}{count}++;
   }
+  push(@{$tests{load}{code}},$t);
 }
 
 sub test_object_creation
 {
   print "Generate object creation tests\n" if $debug;
   $counter = 1;
-  #	create
+  #     create
   #
-  #	Generate tests for object creation
+  #     Generate tests for object creation
   #
   # retrieve list of ellipsoids
   print "Ellipsoids:\n" if $print;
@@ -149,7 +155,7 @@ sub test_object_creation
     printf "  %20s  %12.3f  %s\n", $ell, $a, $rf if $print;
     write_create_test($ell) if $write;
   }
-  
+
   # test custom ellipsoid
   my $custom = Geo::Ellipsoid->new();
   my $name = 'CUSTOM';
@@ -157,15 +163,15 @@ sub test_object_creation
   my $recip = 300;
   my $var = '$e'.$counter++;
   $custom->set_custom_ellipsoid($name,$major,$recip);
-  
+
   my $code = "my $var = Geo::Ellipsoid->new();\n" .
   "${var}->set_custom_ellipsoid('$name',$major,$recip);";
   push( @{${tests{create}}{code}}, $code);
   write_create_test_code($name,$var,$custom);
 
   # warn user about upcoming warning message
-  push( @{${tests{create}}{code}}, 
-    qq(print STDERR "\\n#\\n#\\tWarning about 'Infinite flattening' OK here\\n#\\n;";)
+  push( @{${tests{create}}{code}},
+    q(diag "\n\n\tWarning about 'Infinite flattening' OK here\n\n";)
   );
 
   my $sphere = Geo::Ellipsoid->new;
@@ -185,34 +191,34 @@ sub test_defaults
 
   my $code = <<'EOS';
 my $e1 = Geo::Ellipsoid->new();
-ok( $e1->{ellipsoid} eq 'WGS84' );
-ok( $e1->{units} eq 'radians' );
-ok( $e1->{distance_units} eq 'meter' );
-ok( $e1->{longitude} == 0 );
-ok( $e1->{latitude} == 1 );
-ok( $e1->{bearing} == 0 );
-$e1->set_defaults( 
+is( $e1->{ellipsoid}, 'WGS84' );
+is( $e1->{angle_unit}, 'radians' );
+is( $e1->{distance_unit}, 'meter' );
+cmp_ok( $e1->{longitude_symmetric}, '==', 0 );
+cmp_ok( $e1->{latitude_symmetric}, '==', 1 );
+cmp_ok( $e1->{bearing_symmetric}, '==', 0 );
+$e1->set_defaults(
   ellipsoid => 'NAD27',
-  units => 'degrees', 
-  distance_units => 'kilometer',
-  longitude => 1,
-  bearing => 1
+  angle_unit => 'degrees',
+  distance_unit => 'kilometer',
+  longitude_symmetric => 1,
+  bearing_symmetric => 1
 );
 my $e2 = Geo::Ellipsoid->new();
-ok( $e2->{ellipsoid} eq 'NAD27' );
-ok( $e2->{units} eq 'degrees' );
-ok( $e2->{distance_units} eq 'kilometer' );
-ok( $e2->{longitude} == 1 );
-ok( $e2->{latitude} == 1 );
-ok( $e2->{bearing} == 1 );
+is( $e2->{ellipsoid}, 'NAD27' );
+is( $e2->{angle_unit}, 'degrees' );
+is( $e2->{distance_unit}, 'kilometer' );
+cmp_ok( $e2->{longitude_symmetric}, '==', 1 );
+cmp_ok( $e2->{latitude_symmetric}, '==', 1 );
+cmp_ok( $e2->{bearing_symmetric}, '==', 1 );
 EOS
   push( @{${tests{defaults}}{code}}, $code);
   ${$tests{defaults}}{count} = 12;
 
   $counter = 3;
-  #	defaults
+  #     defaults
   #
-  #	Generate tests for setting default values
+  #     Generate tests for setting default values
   #
   # retrieve list of ellipsoids
   foreach my $ell ( @ellipsoids ) {
@@ -225,29 +231,34 @@ sub write_defaults_test
 {
   my( $ell ) = @_;
   my $var = '$e'.$counter++;
-  my $e = Geo::Ellipsoid->new( ellipsoid => $ell, units => 'degrees' );
+  my $e = Geo::Ellipsoid->new( ellipsoid => $ell, angle_unit => 'degrees' );
   my $a = $e->{equatorial};
   my $b = $e->{polar};
   my $f = $e->{flattening};
   print "  e=$ell, var=$var, a=$a, b=$b, f=$f\n" if $debug;
 
   my $code = <<EOS;
-Geo::Ellipsoid->set_defaults(units=>'degrees',ell=>'$ell');
-ok( \$Geo::Ellipsoid::defaults{ellipsoid} eq '$ell' );
-ok( \$Geo::Ellipsoid::defaults{units} eq 'degrees' );
+Geo::Ellipsoid->set_defaults(angle_unit => 'degrees', ell => '$ell');
+is( \$Geo::Ellipsoid::defaults{ellipsoid}, '$ell',
+    'default ellipsoid is "$ell"' );
+is( \$Geo::Ellipsoid::defaults{angle_unit}, 'degrees',
+    'default angle unit is "degrees"' );
 my $var = Geo::Ellipsoid->new();
-ok( defined ${var} );
-ok( ${var}->isa( 'Geo::Ellipsoid' ) );
-ok( ${var}->{ellipsoid} eq '$ell' );
-ok( ${var}->{units} eq 'degrees' );
-delta_ok( ${var}->{equatorial}, $a );
-delta_ok( ${var}->{polar}, $b );
-delta_ok( ${var}->{flattening}, $f );
+isnt( ${var}, undef, 'object is defined' );
+isa_ok( ${var}, 'Geo::Ellipsoid' );
+is( ${var}->{ellipsoid}, '$ell', 'ellipsoid is "$ell"' );
+is( ${var}->{angle_unit}, 'degrees', 'angle unit is "degrees"' );
+delta_ok( ${var}->{equatorial}, $a,
+    'equatorial radius is within tolerance' );
+delta_ok( ${var}->{polar}, $b,
+    'polar radius is within tolerance' );
+delta_ok( ${var}->{flattening}, $f,
+    'flattening is within tolerance' );
 EOS
   push( @{${tests{defaults}}{code}}, $code );
   ${$tests{defaults}}{count} += 9;
 }
- 
+
 sub test_inverse
 {
   print "Generate inverse tests\n" if $debug;
@@ -256,25 +267,25 @@ sub test_inverse
   my $latinc = 88;
   my $lon0 = 1;
   my $loninc = 89;
-  $e->set_units('degrees');
-  my $code = q|my $e = Geo::Ellipsoid->new(units=>'degrees');|;
+  $e->set_angle_unit('degrees');
+  my $code = q|my $e = Geo::Ellipsoid->new(angle_unit => 'degrees');|;
   $tests{to}{code} = [ $code, 'my( $r, $a );' ];
   ${$tests{to}}{count} = 0;
 
   $code = <<'EOS';
-my $e_pos = Geo::Ellipsoid->new(units=>'degrees');
-my $e_sym = Geo::Ellipsoid->new(units=>'degrees',bearing=>1);
+my $e_pos = Geo::Ellipsoid->new(angle_unit => 'degrees');
+my $e_sym = Geo::Ellipsoid->new(angle_unit => 'degrees',bearing_symmetric => 1);
 my($azp,$azs);
 EOS
   $tests{bearing}{code} = [ $code ];
   ${$tests{bearing}}{count} = 0;
 
   $code = <<'EOS';
-my $e_meter = Geo::Ellipsoid->new(units=>'degrees');
-my $e_kilo = Geo::Ellipsoid->new(units=>'degrees',distance=>'kilo');
-my $e_mile = Geo::Ellipsoid->new(units=>'degrees',distance=>'mile');
-my $e_foot = Geo::Ellipsoid->new(units=>'degrees',distance=>'foot');
-my $e_nm = Geo::Ellipsoid->new(units=>'degrees',distance=>'nm');
+my $e_meter = Geo::Ellipsoid->new(angle_unit => 'degrees');
+my $e_kilo = Geo::Ellipsoid->new(angle_unit => 'degrees',distance => 'kilo');
+my $e_mile = Geo::Ellipsoid->new(angle_unit => 'degrees',distance => 'mile');
+my $e_foot = Geo::Ellipsoid->new(angle_unit => 'degrees',distance => 'foot');
+my $e_nm = Geo::Ellipsoid->new(angle_unit => 'degrees',distance => 'nm');
 my( $r1,$r2,$r3,$r4,$r5);
 EOS
   $tests{range}{code} = [ $code ];
@@ -291,7 +302,7 @@ EOS
 
           # skip tests where points are anti-podal
           next if $lat2 == -$lat1 and abs($lon1-$lon2) == 180;
-          
+
           my( $r, $az ) = $e->to($lat1,$lon1,$lat2,$lon2);
           print "$n: ($lat1,$lon1)->($lat2,$lon2): ($r,$az)\n" if $debug;
           $n++;
@@ -303,7 +314,7 @@ EOS
       }
     }
   }
-  
+
   # test random values
   for ( 1..100 ) {
     my $lat1 = -88 + rand 176;
@@ -312,8 +323,8 @@ EOS
     my $lon2 = rand 360;
 
     # skip tests where points are anti-podal
-    next if (abs($lat2 + $lat1) < 1) and 
-    	    (abs($lon1 - $lon2 - 180) < 1);
+    next if (abs($lat2 + $lat1) < 1) and
+            (abs($lon1 - $lon2 - 180) < 1);
 
     my ( $r, $az ) = $e->to($lat1,$lon1,$lat2,$lon2);
     print "$n: ($lat1,$lon1)->($lat2,$lon2)\n" if $debug;
@@ -329,22 +340,22 @@ sub test_to
   print "Generate to tests\n" if $debug;
   my( $range, $bearing, $lat1, $lon1, $lat2, $lon2 ) = @_;
   my $n = 1;
-  my $l1 = sprintf "%.6f, %.6f", $lat1, $lon1; 
+  my $l1 = sprintf "%.6f, %.6f", $lat1, $lon1;
   my $l2 = sprintf "%.6f, %.6f", $lat2, $lon2;
-  
+
   my $t = sprintf "(\$r,\$a) = \$e->to( %.6f, %.6f, %.6f, %.6f );",
     $lat1, $lon1, $lat2, $lon2;
-    
+
   my $code = "( \$r, \$a ) = \$e->to($l1,$l2);\n";
   if( $range < 100.0 ) {
-    $code .= "delta_within( \$r, $range, 0.1 );\n";
+    $code .= "delta_within( \$r, $range, 0.1, 'range is within tolerance' );\n";
   }else{
-    $code .= "delta_ok( \$r, $range );\n";
+    $code .= "delta_ok( \$r, $range, 'range is within tolerance' );\n";
   }
 
   # if two locations are not the same, test the bearing angle
   if( $l1 ne $l2 ) {
-    $code .= "delta_within( \$a, $bearing, 0.0001 );\n";
+    $code .= "delta_within( \$a, $bearing, 0.0001, 'bearing is within tolerance' );\n";
     $n++;
   }
   push( @{$tests{to}{code}}, $code );
@@ -367,11 +378,11 @@ sub test_range
 \$r3 = \$e_mile->$m
 \$r4 = \$e_foot->$m
 \$r5 = \$e_nm->$m
-delta_within( \$r1, $range, 1.0 );
-delta_within( \$r2, $r2, 1.0 );
-delta_within( \$r3, $r3, 1.0 );
-delta_within( \$r4, $r4, 1.0 );
-delta_within( \$r5, $r5, 1.0 );
+delta_within( \$r1, $range, 1.0, 'range is within tolerance' );
+delta_within( \$r2, $r2, 1.0, 'range is within tolerance' );
+delta_within( \$r3, $r3, 1.0, 'range is within tolerance' );
+delta_within( \$r4, $r4, 1.0, 'range is within tolerance' );
+delta_within( \$r5, $r5, 1.0, 'range is within tolerance' );
 EOS
   push( @{$tests{range}{code}}, $code );
   ${$tests{range}}{count} += 5;
@@ -383,7 +394,7 @@ sub test_bearing
   printf "test_bearing([%.2f,%.2f]->[%.2f,%.2f])\n",
     $lat1, $lon1, $lat2, $lon2 if $debug;
 
-  my $l1 = sprintf "%.6f,%.6f", $lat1, $lon1; 
+  my $l1 = sprintf "%.6f,%.6f", $lat1, $lon1;
   my $l2 = sprintf "%.6f,%.6f", $lat2, $lon2;
   return if $l1 eq $l2;
 
@@ -396,7 +407,7 @@ sub test_bearing
   if( abs($bp) > 0.1 && abs($bp-360) > 0.1 ) {
     my $code = <<EOS;
 \$azp = \$e_pos->bearing($l1,$l2);
-delta_within( \$azp, $bp, 0.1 );
+delta_within( \$azp, $bp, 0.1, 'bearing is within tolerance' );
 EOS
     push( @{$tests{bearing}{code}}, $code );
     ${$tests{bearing}}{count} += 1;
@@ -406,7 +417,7 @@ EOS
   if( abs($bs-180) > 0.1 && abs($bs+180) > 0.1 ) {
     my $code = <<EOS;
 \$azs = \$e_sym->bearing($l1,$l2);
-delta_within( \$azs, $bs, 0.1 );
+delta_within( \$azs, $bs, 0.1, 'bearing is within tolerance' );
 EOS
     push( @{$tests{bearing}{code}}, $code );
     ${$tests{bearing}}{count} += 1;
@@ -417,10 +428,10 @@ sub test_forward
 {
   print "Generate forward tests\n" if $debug;
   my $n = 1;
-  $e->set_units('degrees');
+  $e->set_angle_unit('degrees');
   my $code = <<'EOS';
-my $e1 = Geo::Ellipsoid->new(units=>'degrees');
-my $e2 = Geo::Ellipsoid->new(units=>'degrees',longitude=>1);
+my $e1 = Geo::Ellipsoid->new(angle_unit => 'degrees');
+my $e2 = Geo::Ellipsoid->new(angle_unit => 'degrees',longitude_symmetric => 1);
 my($lat1,$lon1,$lat2,$lon2,$x,$y);
 EOS
 
@@ -440,10 +451,10 @@ EOS
     my $range = rand 10000;
     my $bearing = rand 360;
     my $az = deg2rad($bearing);
-    
+
     my $x = $range * sin($az);
     my $y = $range * cos($az);
-    
+
     my ( $lat2, $lon2 ) = $e->at($lat1,$lon1,$range,$bearing);
     print "$n: ($lat1,$lon1,\n  $range,$bearing,\n  $x,$y)->\n  ($lat2,$lon2)\n"
       if $debug;
@@ -466,10 +477,10 @@ sub test_at
   my $code = <<EOS;
 (\$lat1,\$lon1) = \$e1->$f;
 (\$lat2,\$lon2) = \$e2->$f;
-delta_ok( \$lat1, $lat3 );
-delta_ok( \$lon1, $lon3 );
-delta_ok( \$lat2, $lat4 );
-delta_ok( \$lon2, $lon4 );
+delta_ok( \$lat1, $lat3, 'latitude is within tolerance' );
+delta_ok( \$lon1, $lon3, 'longitude is within tolerance' );
+delta_ok( \$lat2, $lat4, 'latitude is within tolerance' );
+delta_ok( \$lon2, $lon4, 'longitude is within tolerance' );
 EOS
   push( @{$tests{at}{code}}, $code );
   ${$tests{at}}{count} += 4;
@@ -485,10 +496,10 @@ sub test_location
   my $code = <<EOS;
 (\$lat1, \$lon1) = \$e1->$f;
 (\$lat2, \$lon2) = \$e2->$f;
-delta_ok( \$lat1, $lat3 );
-delta_ok( \$lon1, $lon3 );
-delta_ok( \$lat2, $lat4 );
-delta_ok( \$lon2, $lon4 );
+delta_ok( \$lat1, $lat3, 'latitude is within tolerance' );
+delta_ok( \$lon1, $lon3, 'longitude is within tolerance' );
+delta_ok( \$lat2, $lat4, 'latitude is within tolerance' );
+delta_ok( \$lon2, $lon4, 'longitude is within tolerance' );
 EOS
   push( @{$tests{location}{code}}, $code );
   ${$tests{location}}{count} += 4;
@@ -504,8 +515,8 @@ sub test_displacement
     $lat1, $lon1, $lat2, $lon2;
   my $code = <<EOS;
 $t
-delta_within( \$x, $x, 1.0 );
-delta_within( \$y, $y, 1.0 );
+delta_within( \$x, $x, 1.0, 'x displacement is within tolerance' );
+delta_within( \$y, $y, 1.0, 'y displacement is within tolerance' );
 EOS
   push( @{$tests{displacement}{code}}, $code );
   ${$tests{displacement}}{count} += 2;
@@ -515,21 +526,21 @@ EOS
 sub test_set
 {
   print "Generate set tests\n" if $debug;
-  print "Test set_units:\n";
+  print "Test set_angle_unit:\n";
   my $code = <<EOS;
 my \$e = Geo::Ellipsoid->new();
 EOS
   push( @{$tests{set}{code}}, $code );
   my $n = 1;
-  test_set_units($n++,'degrees');
-  test_set_units($n++,'radians');
-  test_set_units($n++,'DEG','degrees');
-  test_set_units($n++,'Deg','degrees');
-  test_set_units($n++,'deg','degrees');
-  test_set_units($n++,'RAD','radians');
-  test_set_units($n++,'Rad','radians');
-  test_set_units($n++,'rad','radians');
-  
+  test_set_angle_unit($n++,'degrees');
+  test_set_angle_unit($n++,'radians');
+  test_set_angle_unit($n++,'DEG','degrees');
+  test_set_angle_unit($n++,'Deg','degrees');
+  test_set_angle_unit($n++,'deg','degrees');
+  test_set_angle_unit($n++,'RAD','radians');
+  test_set_angle_unit($n++,'Rad','radians');
+  test_set_angle_unit($n++,'rad','radians');
+
   for my $ell ( sort @ellipsoids ) {
     test_set_ellipsoid($n++,$ell) unless $ell =~ /custom/i;
   }
@@ -545,42 +556,48 @@ sub test_set_ellipsoid
 my \$e$n = Geo::Ellipsoid->new();
 \$e->set_ellipsoid('$ell');
 \$e${n}->set_ellipsoid('$ell');
-ok( \$e->{ellipsoid} eq '$ell' );
-ok( \$e${n}->{ellipsoid} eq '$ell' );
-delta_ok( \$e${n}->{equatorial}, $e->{equatorial} ); 
-delta_ok( \$e${n}->{polar}, $e->{polar} ); 
-delta_ok( \$e${n}->{flattening}, $e->{flattening} ); 
-delta_ok( \$e->{equatorial}, $e->{equatorial} ); 
-delta_ok( \$e->{polar}, $e->{polar} ); 
-delta_ok( \$e->{flattening}, $e->{flattening} ); 
+is( \$e->{ellipsoid}, '$ell', 'ellipsoid is "$ell"' );
+is( \$e${n}->{ellipsoid}, '$ell', 'ellipsoid is "$ell"' );
+delta_ok( \$e${n}->{equatorial}, $e->{equatorial},
+    'equatorial radius is with tolerance' );
+delta_ok( \$e${n}->{polar}, $e->{polar},
+    'polar radius is with tolerance' );
+delta_ok( \$e${n}->{flattening}, $e->{flattening},
+    'flattening is with tolerance' );
+delta_ok( \$e->{equatorial}, $e->{equatorial},
+    'equatorial radius is with tolerance' );
+delta_ok( \$e->{polar}, $e->{polar},
+    'polar radius is with tolerance' );
+delta_ok( \$e->{flattening}, $e->{flattening},
+    'flattening is with tolerance' );
 EOS
   push(@{$tests{set}{code}}, $code);
   ${$tests{set}}{count} += 8;
 }
-  
-sub test_set_units
+
+sub test_set_angle_unit
 {
-  print "Generate set units tests\n" if $debug;
-  my( $n, $units, $default ) = @_;
-  $default = $units unless $default;
-  print "test set_units($n,$units,$default)\n" if $debug;
+  print "Generate set angle_unit tests\n" if $debug;
+  my( $n, $angle_unit, $default ) = @_;
+  $default = $angle_unit unless $default;
+  print "test set_angle_unit($n,$angle_unit,$default)\n" if $debug;
   my $code = <<EOS;
 my \$e$n = Geo::Ellipsoid->new();
-\$e->set_units('$units');
-\$e${n}->set_units('$units');
-ok( \$e->{units} eq '$default' );
-ok( \$e${n}->{units} eq '$default' );
+\$e->set_angle_unit('$angle_unit');
+\$e${n}->set_angle_unit('$angle_unit');
+is( \$e->{angle_unit}, '$default', 'angle unit is "$default"' );
+is( \$e${n}->{angle_unit}, '$default', 'angle unit is "$default"' );
 EOS
   push(@{$tests{set}{code}}, $code);
   ${$tests{set}}{count} += 2;
 }
-  
+
 sub write_create_test
 {
   my( $ell ) = @_;
   my $var = '$e'.$counter++;
   my $e = Geo::Ellipsoid->new( ellipsoid => $ell );
-  my $t = "my $var = Geo::Ellipsoid->new(ell=>'$ell');";
+  my $t = "my $var = Geo::Ellipsoid->new(ell => '$ell');";
   push( @{${tests{create}}{code}}, $t);
   write_create_test_code($ell,$var,$e);
 }
@@ -593,22 +610,31 @@ sub write_create_test_code
   my $f = $e->{flattening};
   print "  e=$ell, var=$var, a=$a, b=$b, f=$f\n" if $debug;
   my $code = <<EOS;
-ok( defined ${var} );
-ok( ${var}->isa( 'Geo::Ellipsoid' ) );
-ok( ${var}->{ellipsoid} eq '$ell' );
-delta_ok( ${var}->{equatorial}, $a );
-delta_ok( ${var}->{polar}, $b );
+isnt( ${var}, undef, 'object is defined' );
+isa_ok( ${var}, 'Geo::Ellipsoid' );
+is( ${var}->{ellipsoid}, '$ell',
+    'ellipsoid is "$ell"' );
+delta_ok( ${var}->{equatorial}, $a,
+    'equatorial radius is within tolerance' );
+delta_ok( ${var}->{polar}, $b,
+    'polar radius is within tolerance' );
 EOS
 
   # use delta_within instead of delta_ok if flattening is zero
   if( $f == 0 ) {
-    $code .= "delta_within( ${var}->{flattening}, $f, 1e-6 );\n";
+      $code .= <<EOS
+delta_within( ${var}->{flattening}, $f, 1e-6,
+    'flattening is within tolerance' );
+EOS
   }else{
-    $code .= "delta_ok( ${var}->{flattening}, $f );\n";
+      $code .= <<EOS
+delta_ok( ${var}->{flattening}, $f,
+    'flattening is within tolerance' );
+EOS
   }
 
   $code .= <<EOS;
-ok( exists \$Geo::Ellipsoid::ellipsoids{'$ell'} );
+ok( exists \$Geo::Ellipsoid::ellipsoids{'$ell'}, 'ellipsoid "$ell" exists' );
 EOS
   push( @{${tests{create}}{code}}, $code );
   ${$tests{create}}{count} += 7;
@@ -635,14 +661,14 @@ sub test_scale_factors
   print "Generate scale factor tests\n" if $debug;
   # test scale factors
   if( $print ) {
-    print 
-    "\nPrint Latitude and Longitude scale factors (meters per degree):\n\n"; 
+    print
+    "\nPrint Latitude and Longitude scale factors (meters per degree):\n\n";
     print "+----------------------------------------+\n";
     print "| Latitude |    F(Lat)    |    F(Lon)    |\n";
     print "|----------|--------------|--------------|\n";
   }
-  $e = Geo::Ellipsoid->new( ellipsoid => 'WGS84', units => 'degrees' );
-  my $code = "my \$e = Geo::Ellipsoid->new( units => 'degrees' );\n" .
+  $e = Geo::Ellipsoid->new( ellipsoid => 'WGS84', angle_unit => 'degrees' );
+  my $code = "my \$e = Geo::Ellipsoid->new( angle_unit => 'degrees' );\n" .
              "my( \$xs, \$ys );";
   push( @{$tests{scale}{code}}, $code );
   for( my $l = 0; $l <= 89; $l++ ) {
@@ -659,8 +685,8 @@ sub print_latlon_scale
   my( $r_lat, $r_lon ) = $e->scales($deg);
   my $code = <<EOS;
 ( \$ys, \$xs ) = \$e->scales($deg);
-delta_ok( \$xs, $r_lon );
-delta_ok( \$ys, $r_lat );
+delta_ok( \$xs, $r_lon, 'x scale is within tolerance' );
+delta_ok( \$ys, $r_lat, 'y scale is within tolerance' );
 EOS
   push( @{$t->{code}}, $code );
   $t->{count} += 2;
@@ -677,8 +703,9 @@ sub write_test_code
   print "Writing test program $file with $n tests...\n";
   open( my $fh, '>', "$file.t" ) or die("Can't create $file.t: $!");
   write_prolog($fh,$t,$n);
-  print $fh join("\n", @{${$tests{$t}}{code}}), "\n";  
-  close($fh);
+  print $fh join("\n", @{${$tests{$t}}{code}})
+    or die("Can't print to $file.t: $!");
+  close($fh) or die("Can't close file $file.t: $!");
 }
 
 sub write_prolog
@@ -686,22 +713,23 @@ sub write_prolog
   my( $fh, $test, $n ) = @_;
   print "write prolog for ($test,$n)\n" if $debug;
   print $fh <<EOS;
-#!/usr/local/bin/perl
+#!perl
 # Test Geo::Ellipsoid $test
+
+use strict;
+use warnings;
+
 use Test::More tests => $n;
 use Test::Number::Delta relative => 1e-6;
 use Geo::Ellipsoid;
-use blib;
-use strict;
-use warnings;
 
 EOS
 }
 
 my $earth = Geo::Ellipsoid->new(
-  units => 'degrees', 
-  ellipsoid => 'WGS84', 
-  debug => $debug 
+  angle_unit => 'degrees',
+  ellipsoid => 'WGS84',
+  debug => $debug
 );
 
 printf "    Equatorial radius = %.10f\n", $earth->{equatorial};
@@ -758,7 +786,7 @@ sub print_vector
   print "(${lat1deg}d ${lat1min}m ${lat1sec})-(" .
     "${lon1deg}d ${lon1min}m ${lon1sec}) ";
   printf "[%.8f,%.8f] to\n", $lat1, $lon1;
-  print 
+  print
   "(${lat2deg}d ${lat2min}m ${lat2sec})-(${lon2deg}d ${lon2min}m ${lon2sec}) ";
   printf "[%.8f,%.8f]\n", $lat2, $lon2;
 
@@ -767,14 +795,14 @@ sub print_vector
 
 sub print_dist
 {
-  my( $lat1, $lon1, $lat2, $lon2 ) = @_;	# in radians
-  my $ellipsoid = Geo::Ellipsoid->new(uni=>'radians',ell=>'WGS84',deb=>$debug);
-  my $ellipsoid2 = Geo::Ellipsoid->new(uni=>'degrees',ell=>'WGS84',deb=>$debug);
+  my( $lat1, $lon1, $lat2, $lon2 ) = @_;        # in radians
+  my $ellipsoid = Geo::Ellipsoid->new(uni => 'radians',ell => 'WGS84',deb => $debug);
+  my $ellipsoid2 = Geo::Ellipsoid->new(uni => 'degrees',ell => 'WGS84',deb => $debug);
   my( $dlat1, $dlon1, $dlat2, $dlon2 ) = map { $_ * $degrees_per_radian } @_;
 
   printf "\nHere  = [%.12f,%.12f]\n", $dlat1, $dlon1;
   printf "There = [%.12f,%.12f]\n", $dlat2, $dlon2;
-  
+
   my @d = $ellipsoid->displacement( $lat1, $lon1, $lat2, $lon2 );
   my @e = $ellipsoid2->displacement( $dlat1, $dlon1, $dlat2, $dlon2 );
   my( $range, $bearing ) = $ellipsoid->to( $lat1, $lon1, $lat2, $lon2 );
@@ -791,8 +819,8 @@ sub print_dist
 
 sub print_target
 {
-  my( $lat1deg, $lat1min, $lat1sec, 
-      $lon1deg, $lon1min, $lon1sec, 
+  my( $lat1deg, $lat1min, $lat1sec,
+      $lon1deg, $lon1min, $lon1sec,
       $range, $degrees ) = @_;
 
   my $lat1 = Angle($lat1deg,$lat1min,$lat1sec);
@@ -800,9 +828,9 @@ sub print_target
 
   my @here = ( $lat1, $lon1 );
   my $ellipsoid = Geo::Ellipsoid->new(
-    units=>'radians',
-    ellip=>'WGS84',
-    debug=>$debug
+    angle_unit => 'radians',
+    ellip => 'WGS84',
+    debug => $debug
   );
 
   my $radians = $degrees / $degrees_per_radian;
@@ -819,17 +847,17 @@ sub print_target
     @there;
 }
 
-#	Angle
+#       Angle
 #
-#	Return angle in radians given the list 
-#	( degree, minute, second, hundreths-of-second)
+#       Return angle in radians given the list
+#       ( degree, minute, second, hundreths-of-second)
 #
 sub Angle
 {
   my $deg = shift || 0;
   my $min = shift || 0;
   my $sec = shift || 0;
-  my $csec = shift || 0;	# optional 100th's of a second
+  my $csec = shift || 0;        # optional 100th's of a second
 
   #print "convert (@_) to angle in radians\n" if $debug;
   my $frac = ( $min + (($sec + ($csec/100))/60))/60;
@@ -851,4 +879,3 @@ sub polar
   my $bearing = $halfpi - atan2($y,$x);
   return ($range, $bearing);
 }
- 
